
#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1  clrLawnGreen
#property indicator_color2  clrRed
#property strict


extern int                RSIDer          = 14;           // Derivative Period
extern int                FastMaPeriod    = 12;           // Fast Macd Ma Period 
extern ENUM_MA_METHOD     FastMaMode      = MODE_EMA;     // Fast Macd Ma Mode 
extern ENUM_APPLIED_PRICE FastMaPrice     = 0;            // Fast Macd Ma Price 
extern int                SlowMaPeriod    = 26;           // Slow Macd Ma Period 
extern ENUM_MA_METHOD     SlowMaMode      = MODE_EMA;     // Slow Macd Ma Mode
extern ENUM_APPLIED_PRICE SlowMaPrice     = 0;            // Slow Macd Ma Price
extern int                SignalMaPeriod  = 9;            // Signal Macd Ma Period 
extern ENUM_MA_METHOD     SignalMaMode    = MODE_EMA;     // Signal Macd Ma Mode 
extern bool               alertsOn        = true;         // alerts on?
extern bool               alertsOnCurrent = false;        // alerts on current (still open) bar?
extern bool               alertsMessage   = true;         // alerts message?
extern bool               alertsSound     = false;        // alerts sound?
extern bool               alertsNotify    = false;        // alerts notification?
extern bool               alertsEmail     = false;        // alerts email?
extern string             soundFile       = "alert2.wav"; // alerts sound file
extern int                upArrowSize     = 2;            // Arrow size up
extern int                dnArrowSize     = 2;            // Arrow size down
extern int                upArrowCode     = 159;          // Arrow code up
extern int                dnArrowCode     = 159;          // Arrow code down
extern double             upArrowGap      = 0.5;          // Gap for up arrow        
extern double             dnArrowGap      = 0.5;          // Gap for down arrow 


double CrossUp[];
double CrossDn[];
double Ema2[];
double macd[];
double sign[];
double osma[];
double trend[];




int init()
{
   IndicatorBuffers(7);
   SetIndexBuffer(0, CrossUp);   SetIndexStyle(0,DRAW_ARROW,0,upArrowSize); SetIndexArrow(0,upArrowCode);
   SetIndexBuffer(1, CrossDn );  SetIndexStyle(1,DRAW_ARROW,0,dnArrowSize); SetIndexArrow(1,dnArrowCode);
   SetIndexBuffer(2, Ema2);
   SetIndexBuffer(3, macd);
   SetIndexBuffer(4, sign);
   SetIndexBuffer(5, osma);
   SetIndexBuffer(6, trend);
   return(0);
  }
int deinit() { return(0); } 

//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit=MathMin(Bars-counted_bars,Bars-1);
   
   //
   //
   //
   //
   //
    
   for(i=limit; i>=0; i--) macd[i] = iMA(NULL,0,FastMaPeriod,0,FastMaMode,FastMaPrice,i)-iMA(NULL,0,SlowMaPeriod,0,SlowMaMode,SlowMaPrice,i);
   for(i=limit; i>=0; i--) sign[i] = iMAOnArray(macd,0,SignalMaPeriod,0,SignalMaMode,i);
   for(i=limit; i>=0; i--)
   {  
      osma[i]     = macd[i]-sign[i];
      double rsi  = iRSI(NULL,0,RSIDer,PRICE_CLOSE,i);
      double ema0 = iEma(rsi ,5,i,0);
      double ema1 = iEma(ema0,3,i,1);
      Ema2[i]     = iEma(ema0,3,i,2) - iSma(ema1,9,i,0);
      CrossUp[i]  = EMPTY_VALUE;
      CrossDn[i]  = EMPTY_VALUE;
      if (i<(Bars-1))
      {
         trend[i] = 0;
         if (Ema2[i]>0 && Ema2[i+1]<0 && osma[i]>0 && osma[i+1]<0) trend[i] = 1;
         if (Ema2[i]<0 && Ema2[i+1]>0 && osma[i]<0 && osma[i+1]>0) trend[i] =-1;
      }
      if (i<Bars-1 && trend[i]!=trend[i+1])
      {
         if (trend[i] ==  1) CrossUp[i] = Low[i] -iATR(NULL,0,15,i)*upArrowGap;
         if (trend[i] == -1) CrossDn[i] = High[i]+iATR(NULL,0,15,i)*dnArrowGap;
      }
      
   } 
   
   //
   //
   //
   //
   //
   
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar]== 1) doAlert(whichBar,"crossing zero up");
         if (trend[whichBar]==-1) doAlert(whichBar,"crossing zero down");
      }
   }     
return(0);
}
  
//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

double workSma[][2];
double iSma(double price, int period, int r, int instanceNo=0)
{
   if (period<=1) return(price);
   if (ArrayRange(workSma,0)!= Bars) ArrayResize(workSma,Bars); r=Bars-r-1; instanceNo *= 2; int k;

   //
   //
   //
   //
   //
      
   workSma[r][instanceNo+0] = price;
   workSma[r][instanceNo+1] = price; for(k=1; k<period && (r-k)>=0; k++) workSma[r][instanceNo+1] += workSma[r-k][instanceNo+0];  
   workSma[r][instanceNo+1] /= 1.0*k;
   return(workSma[r][instanceNo+1]);
}

//
//
//
//
//

double workEma[][3];
double iEma(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= Bars) ArrayResize(workEma,Bars); r=Bars-r-1; if (r<2) { workEma[r][instanceNo] = price; return(price); }

   //
   //
   //
   //
   //
      
   double alpha = 2.0 / (1.0+period);
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+alpha*(price-workEma[r-1][instanceNo]);
   return(workEma[r][instanceNo]);
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Derivative_Macd ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(), Period(), " Derivative_Macd "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}



